local folder, core = ...

if not core.LibNameplate then
	return
end

--Globals
local pairs = pairs
local table_insert	= table.insert
local table_sort	= table.sort
local table_getn	= table.getn
local Debug = core.Debug

-- Isolate the environment
--~ setfenv(1, core)
-- local

local defaultSettings = core.defaultSettings
local L = core.L or LibStub("AceLocale-3.0"):GetLocale(folder, true)

local P = {}
local prev_OnEnable = core.OnEnable;
function core:OnEnable()
	prev_OnEnable(self);

--~ 	self:Debug("OnEnable", "options", self.db.profile)
	P = self.db.profile
	
	self:BuildSpellUI()
end

local minIconSize = 16
local maxIconSize = 80
local minTextSize = 6
local maxTextSize = 20


--~ defaultSettings.profile.defaultSpells			= 1
defaultSettings.profile.defaultBuffShow			= 1
defaultSettings.profile.defaultDebuffShow			= 1

defaultSettings.profile.unknownSpellDataIcon	= true
defaultSettings.profile.showBuffs				= true
defaultSettings.profile.showDefuffs				= true
--~ defaultSettings.profile.showAura				= false
defaultSettings.profile.saveNameToGUID			= true
defaultSettings.profile.watchCombatlog			= true


defaultSettings.profile.abovePlayers		= true
defaultSettings.profile.aboveNPC		= true
defaultSettings.profile.aboveFriendly		= true
defaultSettings.profile.aboveNeutral		= true
defaultSettings.profile.aboveHostile		= true

defaultSettings.profile.barAnchorPoint		= "BOTTOM"
defaultSettings.profile.plateAnchorPoint		= "TOP"
defaultSettings.profile.barOffsetX		= 0
defaultSettings.profile.barOffsetY		= 8 --bit north incase user's running Threat plates.
defaultSettings.profile.iconsPerBar		= 6
defaultSettings.profile.barGrowth		= 1
defaultSettings.profile.numBars		= 2
defaultSettings.profile.iconSize		= 24
defaultSettings.profile.biggerSelfSpells		= true
defaultSettings.profile.showCooldown		= true
defaultSettings.profile.shrinkBar		= true
defaultSettings.profile.cooldownSize		= 10
defaultSettings.profile.stackSize		= 10

defaultSettings.profile.blinkTimeleft		= 0.2 --20%
defaultSettings.profile.showTotems = false



core.CoreOptionsTable = {
	name = core.titleFull,
	type = "group",
	childGroups = "tab",

	args = {
	
		enable = {
			type = "toggle",	order	= 1,
			name	= L["Enable"],
			desc	= L["Enables / Disables the addon"],
			set = function(info,val) 
				if val == true then
					core:Enable()
				else
					core:Disable()
				end
			end,
			get = function(info) return core:IsEnabled() end
		},

		unknownSpellDataIcon = {
			type = "toggle", order	= 10,
			name = L["Unknown spell info"],
			desc = L["Display a question mark above plates we don't know spells for. Target or mouseover those plates."],

			set = function(info,val) 
				P.unknownSpellDataIcon = val
			end,
			get = function(info) return P.unknownSpellDataIcon end
		},
		
		defaultBuffShow = {
			type = "select", order = 11,
			name = L["Show Buffs"],
			desc = L["Show buffs above nameplate."],

			values = function()
				return {
					L["All"], 
					L["Mine Only"],
					L["None"],
				}
			end,
			set = function(info,val) 
				P.defaultBuffShow = val
			end,
			get = function(info) return P.defaultBuffShow end
		},
		
		defaultDebuffShow = {
			type = "select", order = 12,
			name = L["Show Debuffs"],
			desc = L["Show debuffs above nameplate."],

			values = function()
				return {
					L["All"], 
					L["Mine Only"],
					L["None"],
				}
			end,
			set = function(info,val) 
				P.defaultDebuffShow = val
			end,
			get = function(info) return P.defaultDebuffShow end
		},

		saveNameToGUID = {
			type = "toggle", order	= 15,
			name = L["Save player GUID"],
			desc = L["Remember player GUID's so target/mouseover isn't needed every time nameplate appears.\nKeep this enabled"],

			set = function(info,val) 
				P.saveNameToGUID = val
			end,
			get = function(info) return P.saveNameToGUID end
		},
		
		watchCombatlog = {
			type = "toggle", order	= 16,
			name = L["Watch Combatlog"],
			desc = L["Watch combatlog for people gaining/losing spells.\nDisable this if you're having performance issues."],

			set = function(info,val) 
				P.watchCombatlog = val
				if val then
					core:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				else
					core:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				end
			end,
			get = function(info) return P.watchCombatlog end
		},
		
		
	},
}



core.WhoOptionsTable = {
	name = core.titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		typeHeader = {
			name	= L["Type"],
			order	= 10,
			type = "header",
		},

		abovePlayers = {
			name = L["Players"],
			desc = L["Add buffs above players"],
			type = "toggle",
			order	= 11,
			set = function(info,val) 
				P.abovePlayers = val
			end,
			get = function(info) return P.abovePlayers end
		},
	
		aboveNPC = {
			name = L["NPC"],
			desc = L["Add buffs above NPCs"],
			type = "toggle",
			order	= 12,
			set = function(info,val) 
				P.aboveNPC = val
			end,
			get = function(info) return P.aboveNPC end
		},
	
		reactionHeader = {
			name	= L["Reaction"],
			order	= 15,
			type = "header",
		},

		aboveFriendly = {
			name = L["Friendly"],
			desc = L["Add buffs above friendly plates"],
			type = "toggle",
			order	= 16,
			set = function(info,val) 
				P.aboveFriendly = val
			end,
			get = function(info) return P.aboveFriendly end
		},

		aboveNeutral = {
			name = L["Neutral"],
			desc = L["Add buffs above neutral plates"],
			type = "toggle",
			order	= 17,
			set = function(info,val) 
				P.aboveNeutral = val
			end,
			get = function(info) return P.aboveNeutral end
		},
		
		aboveHostile = {
			name = L["Hostile"],
			desc = L["Add buffs above hostile plates"],
			type = "toggle",
			order	= 18,
			set = function(info,val) 
				P.aboveHostile = val
			end,
			get = function(info) return P.aboveHostile end
		},

		otherHeader = {
			name	= L["Other"],
			order	= 20,
			type = "header",
		},

		showTotems = {
			name = L["Show Totems"],
			desc = L["Show spell icons on totems"],
			type = "toggle",
			order	= 21,
			set = function(info,val) 
				P.showTotems = val
			end,
			get = function(info) return P.showTotems end
		},
	},
}

core.BarOptionsTable = {
	name = core.titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		barAnchorPoint = {
			name = L["Bar Anchor Point"],
			order = 1,
			desc = L["Point of the buff frame that gets anchored to the nameplate.\ndefault = Bottom"],
			type = "select",
			
			values = function()
				return {
					["TOP"]=L["Top"], 
					["BOTTOM"]=L["Bottom"], 
--~ 					["LEFT"]=L["Left"], --NOTE! Using these values may break bar anchors to other bars.
--~ 					["RIGHT"]=L["Right"], 
--~ 					["CENTER"]=L["Center"], 
					["TOPLEFT"]=L["Top Left"], 
					["BOTTOMLEFT"]=L["Bottom Left"], 
					["TOPRIGHT"]=L["Top Right"], 
					["BOTTOMRIGHT"]=L["Bottom Right"], 
				}
			end,
			set = function(info,val) 
				P.barAnchorPoint = val
				core:ResetAllBarPoints()
			end,
			get = function(info) return P.barAnchorPoint end
		},

		plateAnchorPoint = {
			name = L["Plate Anchor Point"],
			order = 2,
			desc = L["Point of the nameplate our buff frame gets anchored to.\ndefault = Top"],
			type = "select",
			
			values = function()
				return {
					["TOP"]=L["Top"], 
					["BOTTOM"]=L["Bottom"], 
--~ 					["LEFT"]=L["Left"],   --NOTE! Using these values may break bar anchors to other bars.
--~ 					["RIGHT"]=L["Right"], 
--~ 					["CENTER"]=L["Center"], 
					["TOPLEFT"]=L["Top Left"], 
					["BOTTOMLEFT"]=L["Bottom Left"], 
					["TOPRIGHT"]=L["Top Right"], 
					["BOTTOMRIGHT"]=L["Bottom Right"], 
				}
			end,
			set = function(info,val) 
				P.plateAnchorPoint = val
				core:ResetAllBarPoints()
			end,
			get = function(info) return P.plateAnchorPoint end
		},
		
		barOffsetX = {
			name = L["Bar X Offset"],
			desc = L["Left to right offset."],
			type = "range",
			order	= 5,
			min		= -100,
			max		= 100,
			step	= 1,
			set = function(info,val) 
				P.barOffsetX = val
				core:ResetAllBarPoints()
			end,
			get = function(info) return P.barOffsetX end
		},

		
		barOffsetY = {
			name = L["Bar Y Offset"],
			desc = L["Up to down offset."],
			type = "range",
			order	= 6,
			min		= -100,
			max		= 100,
			step	= 1,
			set = function(info,val) 
				P.barOffsetY = val
				core:ResetAllBarPoints()
			end,
			get = function(info) return P.barOffsetY end
		},
		
		iconsPerBar = {
			name = L["Icons per bar"],
			desc = L["Number of icons to display per bar."],
			type = "range",
			order	= 7,
			min		= 1,
			max		= 16,
			step	= 1,
			set = function(info,val) 
				P.iconsPerBar = val
				core:ResetAllPlateIcons()
		--~ 		core:UpdateBarsSize()
				core:ShowAllKnownSpells()
			end,
			get = function(info) return P.iconsPerBar end
		},
		
		barGrowth = {
			type = "select",	order = 8,
			name = L["Bar Growth"],
			desc = L["Which way do the bars grow, up or down."],

			values = function()
				return {
					L["Up"],
					L["Down"],
				}
			end,
			set = function(info,val) 
				P.barGrowth = val
		--~ 		core:ResetAllPlateIcons()
				core:ResetAllBarPoints()
			end,
			get = function(info) return P.barGrowth end
		},
		
		numBars = {
			name = L["Max bars"],
			desc = L["Max number of bars to show."],
			type = "range",
			order	= 9,
			min		= 1,
			max		= 4,
			step	= 1,
			set = function(info,val) 
				P.numBars = val
				core:ResetAllPlateIcons()
		--~ 		core:UpdateBarsSize()
				core:UpdateBarsBackground()
				core:ShowAllKnownSpells()
			end,
			get = function(info) return P.numBars end
		},
		
		
		
		
		
		biggerSelfSpells = {
			name = L["Larger self spells"],
			desc = L["Make your spells 20% bigger then other's."],
			type = "toggle",
			order	= 11,
			set = function(info,val) 
				P.biggerSelfSpells = val
			end,
			get = function(info) return P.biggerSelfSpells end
		},
		
		
		showCooldown = {
			name = L["Show cooldown"],
			desc = L["Show cooldown text under the spell icon."],
			type = "toggle",
			order	= 12,
			set = function(info,val) 
				P.showCooldown = val
		--~ 		core:ResetCooldownSize()
		--~ 		core:ResetAllPlateIcons()
				core:ResetIconSizes()--Adjust the frame height with the new cooldown size
		--~ 		core:UpdateBarsSize()
				core:ShowAllKnownSpells()
			end,
			get = function(info) return P.showCooldown end
		},
		

		shrinkBar = {
			name = L["Shrink Bar"],
			desc = L["Shrink the bar horizontally when spells frames are hidden."],
			type = "toggle",
			order	= 20,
			set = function(info,val) 
				P.shrinkBar = val
				core:UpdateAllPlateBarSizes()
			end,
			get = function(info) return P.shrinkBar end
		},
		
		showBarBackground = {
			name = L["Show bar background"],
			desc = L["Show the area where spell icons will be. This is to help you configure the bars."],
			type = "toggle",
			order	= 99,
			set = function(info,val) 
				P.showBarBackground = val
				core:UpdateBarsBackground()
			end,
			get = function(info) return P.showBarBackground end
		},
		
		
		iconTestMode = {
			name = L["Test Mode"],
			desc = L["For each spell on someone, multiply it by the number of icons per bar.\nThis option won't be saved at logout."],
			type = "toggle",
			order	= 100,
			set = function(info,val) 
				core.iconTestMode = val
			end,
			get = function(info) return core.iconTestMode end
		},
		
	},
}

defaultSettings.profile.spellOpts = {}

-- END OF BAR

local tmpNewName = ""
core.SpellOptionsTable = {
	name = core.titleFull,
	type = "group",
	args = {
--~ 				whatThisDo = {
--~ 					type = "description",	order = 5,
--~ 					name = "Specific spell options.",
--~ 				},

		
		inputName = {
			type = "input",	order	= 2,
			name = L["Spell name"],
			desc = L["Input a spell name. (case sensitive)\nOr spellID"],
			set = function(info,val) 
				local num = tonumber(val)
				if num then
					local spellName = GetSpellInfo(num)
--~ 					Debug("num 1", num, spellName)
					if spellName then
						tmpNewName = spellName
						return
					end
				end
--~ 				Debug(val, type(val))
				tmpNewName = val
			end,
			get = function(info) return tmpNewName end
		},

		addName = {
			type = "execute",	order	= 3,
			name	= L["Add spell"],
			desc	= L["Add spell to list."],
			func = function(info) 
				if tmpNewName ~= "" then
					core:AddNewSpell(tmpNewName)
					tmpNewName = ""
				end
			end
		},

		defaultSpellOpt = {
			type = "group",	order	= 4,
			name	= "Default",
			args={
				iconSize = {
					name = L["Icon Size"],
					desc = L["Size of the icons."],
					type = "range",
					order	= 10,
					min		= minIconSize,
					max		= maxIconSize,
					step	= 4,
					set = function(info,val) 
						P.iconSize = val
						core:ResetIconSizes()
				--~ 		core:UpdateBarsSize()
					end,
					get = function(info) return P.iconSize end
				},
			
				cooldownSize = {
					name = L["Cooldown Size"],
					desc = L["Text size"],
					type = "range",
					order	= 13,
					min		= minTextSize,
					max		= maxTextSize,
					step	= 1,
					set = function(info,val) 
						P.cooldownSize = val
						
						core:ResetCooldownSize()
						core:ResetAllPlateIcons()
						core:ResetIconSizes()--Adjust the frame height with the new cooldown size
				--~ 		core:UpdateBarsSize()
						core:ShowAllKnownSpells()
					end,
					get = function(info) return P.cooldownSize end
				},
		
				stackSize = {
					name = L["Stack Size"],
					desc = L["Text size"],
					type = "range",
					order	= 14,
					min		= minTextSize,
					max		= maxTextSize,
					step	= 1,
					set = function(info,val) 
						P.stackSize = val
						core:ResetStackSizes()
					end,
					get = function(info) return P.stackSize end
				},
			
				blinkTimeleft = {
					name = L["Blink Timeleft"],
					desc = L["Blink spell if below x% timeleft, (only if it's below 60 seconds)"],
					type = "range",
					order	= 15,
					min		= 0,
					max		= 1,
					step	= 0.05,
					isPercent = true,
					set = function(info,val) 
						P.blinkTimeleft = val
					end,
					get = function(info) return P.blinkTimeleft end
				},

			}
		},

		spellList = {
			type = "group",	order	= 6,
			name	= L["Specific"],
			args={}, --done later
		},

		
		
		
	},
}




function core:BuildSpellUI()
	local SpellOptionsTable = core.SpellOptionsTable
	SpellOptionsTable.args.spellList.args = {} --reset

	local list = {}
	for name, data in pairs(P.spellOpts) do 
		table_insert(list, name)
	end
	
	table_sort(list, function(a,b) 
		if(a and b) then 
			return a < b
		end 
	end)
	
	local spellName, data
	for i=1, table_getn(list) do 
		spellName = list[i]
		data = P.spellOpts[spellName]
		
		--add spell to table.
		SpellOptionsTable.args.spellList.args[spellName] = {
			type	= "group",
			name	= spellName,
			desc	= L["Spell name"],
			order = i,
			args={}
		}
		
		--UI elements
		SpellOptionsTable.args.spellList.args[spellName].args.showOpt = {
			type = "select",	order = 2,
			name = L["Show"],
			desc = L["Always show spell, only show your spell, never show spell"],

			values = {
				L["Always"],
				L["Mine only"],
				L["Never"],
			},
			set = function(info,val) 
				P.spellOpts[info[2]].show = val
			end,
			get = function(info) return P.spellOpts[info[2]].show or 1 end
		}
		
		
		SpellOptionsTable.args.spellList.args[spellName].args.iconSize = {
			name = L["Icon Size"],
			desc = L["Size of the icons."],
			type = "range",
			order	= 3,
			min		= minIconSize,
			max		= maxIconSize,
			step	= 4,
			set = function(info,val) 
				P.spellOpts[info[2]].iconSize = val
				
				core:ResetIconSizes()
			end,
			get = function(info) return P.spellOpts[info[2]].iconSize or P.iconSize end
		}
		
		SpellOptionsTable.args.spellList.args[spellName].args.cooldownSize = {
			name = L["Cooldown Size"],
			desc = L["Text size"],
			type = "range",
			order	= 4,
			min		= minTextSize,
			max		= maxTextSize,
			step	= 1,
			set = function(info,val) 
				P.spellOpts[info[2]].cooldownSize = val
				
				core:ResetCooldownSize()
				core:ResetAllPlateIcons()
				core:ResetIconSizes()--Adjust the frame height with the new cooldown size
		--~ 		core:UpdateBarsSize()
				core:ShowAllKnownSpells()
			end,
			get = function(info) return P.spellOpts[info[2]].cooldownSize or P.cooldownSize end
		}
		
		SpellOptionsTable.args.spellList.args[spellName].args.stackSize = {
			name = L["Stack Size"],
			desc = L["Text size"],
			type = "range",
			order	= 14,
			min		= minTextSize,
			max		= maxTextSize,
			step	= 1,
			set = function(info,val) 
				P.spellOpts[info[2]].stackSize = val
				core:ResetStackSizes()
			end,
			get = function(info) return P.spellOpts[info[2]].stackSize or P.stackSize end
		}
		
		
		--last
		
		SpellOptionsTable.args.spellList.args[spellName].args.addedWhen = {
			type = "description",	order = 90,
			name = L["Added: "]..data.when,
		}
		
		
		SpellOptionsTable.args.spellList.args[spellName].args.removeSpell = {
			type = "execute",	order	= 91,
			name	= L["Remove Spell"],
			desc	= L["Remove spell from list"],
			func = function(info) 
				core:RemoveSpell(info[2])
			end
		}
		
	end
	
	
end
